<?php

class Universal_email_preference_center_Admin
{
    private  $plugin_name ;
    private  $version ;
    public  $email_centers ;
    public  $centre_type ;
    public $premium;

    public function __construct( $plugin_name, $version )
    {
        $this->plugin_name = $plugin_name;
        $this->version = $version;
        $this->email_centers = [
            "active_campaign" => [
                "name"  => "Active Campaign",
                "route" => "/api/3/",
            ],
                "iterable"        => [
                "name"  => "Iterable",
                "route" => "/api/",
            ],
        ];
        $this->centre_type = esc_attr( get_option( $this->plugin_name . '_centre_type' ) );
        
        if ( defined("UEPC_PREMIUM_DIR_PATH") && file_exists( UEPC_PREMIUM_DIR_PATH.'/admin/admin.php' ) ) {
            require_once UEPC_PREMIUM_DIR_PATH . '/admin/admin.php';

            $this->premium = new Universal_email_preference_center_Admin_Premium($this->plugin_name, $this->version);
        }
    }
    
    public function enqueue_scripts()
    {
        wp_enqueue_script('jquery-ui-dialog');
        wp_enqueue_style('wp-jquery-ui-dialog');
                
        wp_enqueue_script(
            $this->plugin_name . '_run_prettify',
            plugin_dir_url( __FILE__ ) . 'js/run_prettify.js',
            array( 'jquery' ),
            $this->version,
            true
        );

        wp_enqueue_script(
            $this->plugin_name,
            plugin_dir_url( __FILE__ ) . 'js/admin.js',
            array( 'jquery' ),
            $this->version,
            false
        );
    }
    
    public function add_plugin_options()
    {
        add_menu_page(
            'Settings Admin',
            'Universal email preference center',
            'manage_options',
            'universal-email-preference-center',
            array( $this, 'create_options_page' ),
            'dashicons-networking',
            70
        );

        add_submenu_page(
            'universal-email-preference-center',
            'Logs',
            'Logs',
            'manage_options',
            'universal-email-preference-center-logs',
            array(
                $this,
                'create_logs_page',
            )
        );
    }
    
    public function create_options_page()
    {
        global  $uepc_freemius ;
        if ( isset( $_REQUEST['settings-updated'] ) && !empty($_REQUEST['settings-updated']) ) {
            echo  '<div class="updated"><p>' . esc_attr_x( 'Your settings were saved!', 'universal-email-preference-center' ) . '</p></div>' ;
        }
        if ( !current_user_can( 'manage_options' ) ) {
            wp_die( __( 'You do not have sufficient permissions to access this page.' ) );
        }
        if ( isset( $_POST['updated'] ) && $_POST['updated'] === 'true' && wp_verify_nonce( $_POST['save_list_data_nonce'], 'save_list_data' ) ) {
            $this->handle_settings_form();
        }
        
        if ( $this->has_credentials() ) {
            $upec = new Universal_email_preference_center();
            $valid = $upec->validate_credentials();
        } else {
            $valid = false;
        }
        
        
        if ( $valid ) {
            $lists = $this->get_lists();
            $list_data = get_option( $this->plugin_name . '_list_data' );
        } else {
            $validation_error = 'Your ' . $this->centre_type . ' API Credentials are not valid.';
            uepc_log($this->centre_type, "Your ".$this->centre_type." API Credentials are not valid.", 'notice');
        }
        
        $tamper_protection_email_subject = get_option( $this->plugin_name . '_tamper_protection_email_subject', _x( 'Manage Your %%BLOG_NAME%% Email Subscription', 'universal-email-preference-center' ) );
        $tamper_protection_email_content = get_option( $this->plugin_name . '_tamper_protection_email_content', _x( "Someone has requested to update your email subscription to %%BLOG_NAME%%.  In order to do so you need to click the link below. \n\n\n %%LINK%% ", 'universal-email-preference-center' ) );
        $tamper_protection_emailing = get_option( $this->plugin_name . '_tamper_protection_emailing', esc_attr_x( 'Please check your email, it contains a link to confirm you have access to manage your subscription.', 'universal-email-preference-center' ) );
        $heading_available_lists = get_option( $this->plugin_name . '_heading_available_lists', esc_attr_x( 'Available Lists', 'universal-email-preference-center' ) );
        $heading_welcome = get_option( $this->plugin_name . '_heading_welcome', _x( 'Welcome %%EMAIL%%, you are subscribed to the following lists.', 'universal-email-preference-center' ) );
        $heading_retrieve = get_option( $this->plugin_name . '_heading_retrieve', esc_attr_x( 'Retrieve Email Preferences', 'universal-email-preference-center' ) );
        $placeholder_email = get_option( $this->plugin_name . '_placeholder_email', esc_attr_x( 'Enter your email...', 'universal-email-preference-center' ) );
        $button_save_preferences = get_option( $this->plugin_name . '_button_save_preferences', esc_attr_x( 'Save Preferences', 'Button text if email has NOT been retrieved', 'universal-email-preference-center' ) );
        $button_subscribe = get_option( $this->plugin_name . '_button_subscribe', esc_attr_x( 'Subscribe', 'Button text if email HAS been retrieved.', 'universal-email-preference-center' ) );
        $button_retrieve = get_option( $this->plugin_name . '_button_retrieve', esc_attr_x( 'Retrieve Email Preferences', 'Button text if email HAS NOT been retrieved.', 'universal-email-preference-center' ) );
        $string_success_text = get_option( $this->plugin_name . '_string_success_text', esc_attr_x( 'Your email preferences have been saved.', 'Success string when subscribing to list', 'universal-email-preference-center' ) );
        $string_error_text = get_option( $this->plugin_name . '_string_error_text', esc_attr_x( 'There was an error processing your request.  Try again.', 'Error string when subscribing to list', 'universal-email-preference-center' ) );
        $string_saving_text = get_option( $this->plugin_name . '_string_saving_text', esc_attr_x( 'Please wait...', 'Text when save preferences button is pressed', 'universal-email-preference-center' ) );
        $name_position = get_option( $this->plugin_name . '_name_position', esc_attr_x( 'bottom', 'universal-email-preference-center' ) );
        $enable_name = filter_var( get_option( $this->plugin_name . '_enable_name' ), FILTER_VALIDATE_BOOLEAN );
        require_once plugin_dir_path( __FILE__ ) . 'partials/settings.php';
    }
    
    /* 
    *
    * LOGS TEMPLATE FUNCTION
    */
    public function create_logs_page(){
		$openFile = '';
		$logAction = isset($_REQUEST['log-action']) ? sanitize_text_field($_REQUEST['log-action']):"";
		if ($logAction == 'view') {
			$openFile = sanitize_text_field($_REQUEST['log-file']);
		} elseif ($logAction == 'delete') {
            if (file_exists(dirname(plugin_dir_path(__FILE__)) . '/logs/' . sanitize_text_field($_REQUEST['log-file']))) {
                unlink( dirname(plugin_dir_path(__FILE__)) . '/logs/' . sanitize_text_field($_REQUEST['log-file']) );
            }
            echo "<script>window.location = '".admin_url('/admin.php?page=universal-email-preference-center-logs')."'</script>";
            return;
		}

		$logFiles = [];
		if ( is_dir(dirname(plugin_dir_path(__FILE__)) . '/logs') && $handle = opendir(dirname(plugin_dir_path(__FILE__)) . '/logs')) {
			$ignore = array('cgi-bin', '.', '..', '._');
			while (false !== ($entry = readdir($handle))) {
				if (!in_array($entry, $ignore) and substr($entry, 0, 1) != '.') {
					$logFiles[] = $entry;
				}
			}
			closedir($handle);
			krsort($logFiles);
			foreach ($logFiles as $key => $entry) {
				$selected = '';
				if ($openFile != '') {
					if ($entry == $openFile) {
						$selected = 'selected';
					}
				} else {
					$selected = (count($logFiles) == $key) ? 'selected' : '';
				}
			}
		}

		if ($openFile == '') {
			$openFile = isset($logFiles[count($logFiles) - 1]) ? $logFiles[count($logFiles) - 1] : "";
		}
		$logFile = dirname(plugin_dir_path(__FILE__)) . '/logs/' . $openFile;
		$logData = '';
		if (file_exists($logFile) && !empty($openFile)) {
			$file = file($logFile);
			$file = array_reverse($file);
			foreach ($file as $f) {
				$logData .= "<button class='accordion'>" . $this->limit_text($f, 8) . "</button>
						<div class='panel'>
							<p>$f</p>
						</div>";
			}
		} else {
			$logData .= "<div class='notice notice-warning is-dismissible'>
					<p>Log's File Is Not Exist </p>
				</div>";
		}

		require_once plugin_dir_path(__FILE__) . 'partials/logs.php';
	}
    
    /**
     * Truncate string as per limit.
     */
    function limit_text( $text, $limit )
    {
        $pos = stripos( $text, '<br>' );
        
        if ( $pos > -1 ) {
            $text = substr( $text, 0, $pos ) . "...";
        } else {
            
            if ( str_word_count( $text, 0 ) > $limit ) {
                $words = str_word_count( $text, 2 );
                $pos = array_keys( $words );
                $text = substr( $text, 0, $pos[$limit] ) . '...';
            }
        
        }
        
        return $text;
    }
    
    public function options_init()
    {
        register_setting( $this->plugin_name . '_api', $this->plugin_name . '_centre_type' );
        register_setting( $this->plugin_name . '_api', $this->plugin_name . '_key' );
        register_setting( $this->plugin_name . '_api', $this->plugin_name . '_url' );
        register_setting( $this->plugin_name . '_lists', $this->plugin_name . '_list_data' );
    }
    
    public function has_credentials()
    {
        $creds = get_option( $this->plugin_name . '_centre_type' );
        $creds = get_option( $this->plugin_name . '_key' );
        $creds = get_option( $this->plugin_name . '_url' );
        
        if ( !$creds ) {
            return false;
        } else {
            return true;
        }
    
    }
    
    public function handle_settings_form()
    {
        if (function_exists("uepcp_fs") && uepcp_fs()->is__premium_only() && uepcp_fs()->can_use_premium_code() && !empty($this->premium)) {
            $data = $this->premium->process_settings_form__premium_only();
        } else {
            $data = $this->process_settings_form();
        }
        $this->save_settings_data( $data );
    }
    
    public function save_settings_data( $data )
    {
        error_log( print_r( $data, true ) );
        
        if ( $data != get_option( $this->plugin_name . '_list_data' ) ) {
            
            if ( update_option( $this->plugin_name . '_list_data', $data, false ) ) {
                echo  '<div class="updated"><p>' . esc_attr_x( 'Your fields were saved!', 'universal-email-preference-center' ) . '</p></div>' ;
            } else {
                uepc_log($this->centre_type, "There was an issue saving your list data.<br>".json_encode($data), 'error');
                echo  '<div class="error"><p>' . esc_attr_x( 'ERROR: There was an issue saving your list data.', 'universal-email-preference-center' ) . '</p></div>' ;
            }
        
        } else {
            echo  '<div class="updated"><p>' . esc_attr_x( 'Your fields were saved!', 'universal-email-preference-center' ) . '</p></div>' ;
        }
    
    }
    
    public function process_settings_form()
    {
        $channel = $_POST['channelNameUpdate'] ?? "";
        $data = array();
        foreach ( $_POST['list_id'] as $list_id ) {
            $data[$list_id] = array(
                'id'          => (int) $list_id,
                'enabled'     => true,
                'name'        => sanitize_text_field( $_POST['name'][$list_id] ),
                'description' => '',
                'channel' => [
                    'id' => isset($_POST['channelId']) ? wp_kses_post($_POST['channelId'][$list_id]) : "",
                    'name' => isset($_POST['channelName']) ? wp_kses_post($_POST['channelName'][$list_id]) : "",
                    'updated_name' => isset($_POST['channelName']) ? $channel[$_POST['channelName'][$list_id]]['name'] : "",
                    'enabled' => isset($_POST['channelName']) && isset($channel[$_POST['channelName'][$list_id]]['enabled']) ? $channel[$_POST['channelName'][$list_id]]['enabled'] : ""
                ],
            );
        }
        return $data;
    }
    
    public function get_lists()
    {
        $upec = new Universal_email_preference_center();
        return $upec->get_lists();
    }
    
    public function plugin_settings_link( $links )
    {
        $url = admin_url( 'admin.php?page=universal-email-preference-center' );
        $_link = '<a href="' . $url . '">' . 'Settings' . '</a>';
        $links[] = $_link;
        return $links;
    }
    
    public function iterable_class_search( $search_key, $search, $array )
    {
        if ( !empty($array) ) {
            foreach ( $array as $value ) {
                if ( isset( $value['channel'] ) && $value['channel'][$search_key] == $search ) {
                    return $value['channel'];
                }
            }
        }
    }

    public function get_pages(){
        global $wp, $wpdb;
        $result = $wpdb->get_results( "SELECT ID, post_title FROM wp_posts WHERE post_content LIKE '%[universal-email-preference-center%' AND post_status='publish'");
        if ($result){
            foreach($result as $item){
                _e('<a target="_blank" href= "'.get_permalink($item->ID).'">'.$item->post_title.'</a><br/>');
            }
        }else{
            echo '<p style="color:red;">Please use shortcode get pages!</p>';
        }
    }

}