<?php

class Universal_email_preference_center
{
	protected $loader;
	protected $plugin_name;
	protected $version;
	public $email_centers;

	public $centre_type;
	public $key_name;

	public function __construct()
	{
		if (defined('UEPC_PLUGIN_VERSION')) {
			$this->version = UEPC_PLUGIN_VERSION;
		} else {
			$this->version = '1.0.0';
		}

		$this->plugin_name = 'universal_email_preference_center';

		$this->centre_type = esc_attr(get_option($this->plugin_name . '_centre_type'));

		switch ($this->centre_type) {
			case 'iterable':
				$this->key_name = "Api-Key";
				break;
			default:
				$this->key_name = "Api-Token";
				break;
		}

		$this->email_centers = [
			"active_campaign" => [
				"name" => "Active Campaign",
				"route" => "/api/3/"
			],
			"iterable" => [
				"name" => "Iterable",
				"route" => "/api/"
			]
		];

		$this->load_dependencies();
		$this->define_admin_hooks();
		$this->define_public_hooks();
	}

	public function get_plugin_name()
	{
		return $this->plugin_name;
	}

	public function get_version()
	{
		return $this->version;
	}

	public function run()
	{
		$this->loader->run();
	}

	public function get_loader()
	{
		return $this->loader;
	}

	public function get_lists()
	{
		$return = array();

		switch ($this->centre_type) {
			case 'iterable':
				$endpoint = "messageTypes";
				break;
			default:
				$endpoint = "lists";
				break;
		}

		$lists  = $this->get($endpoint, array('limit' => '100'));
		switch ($this->centre_type) {
			case 'iterable':

				$channels = $this->get('channels');
				if (!empty($channels)) {
					$channels = $channels->channels;
				}

				foreach ($lists->messageTypes as $list) {
					if (is_object($list)) {
						$key = array_search($list->channelId, array_column($channels, 'id'));
						$channelName = $channels[$key]->name;
						$return[$channelName][$list->id] = (array)$list;
					}
				}
				break;
			default:
				foreach ($lists->lists as $list) {
					if (is_object($list)) {
						$return[$list->id] = array(
							'id'   => $list->id,
							'name' => $list->name
						);
					}
				}
				break;
		}

		return $return;
	}

	public function get_contact($email)
	{
		switch ($this->centre_type) {
			case 'iterable':
				$contact = $this->get_iterable_contact($email);
				break;
			default:
				$contact = $this->get_ac_contact($email);
				break;
		}

		return $contact;
	}

	public function get_ac_contact($email)
	{
		$contact = $this->get('contacts', array('email' => $email));

		if (empty($contact->contacts)) {
			$contact = $this->create_contact(sanitize_email($email));

			$return = [
				'id'         => $contact->id,
				'first_name' => $contact->firstName,
				'last_name'  => $contact->lastName,
				'email'      => $contact->email
			];

			$list_url = $contact->links->contactLists;
		} else {
			$return = [
				'id'         => $contact->contacts[0]->id,
				'first_name' => $contact->contacts[0]->firstName,
				'last_name'  => $contact->contacts[0]->lastName,
				'email'      => $contact->contacts[0]->email
			];

			$list_url = $contact->contacts[0]->links->contactLists;
		}

		$get_lists = $this->get($list_url);

		$lists = [];

		if (!is_null($get_lists)) {
			foreach ($get_lists->contactLists as $list) {
				if ($list->status === '1') {
					$lists[] = $list->list;
				}
			}
		}

		$return['lists'] = $lists;

		return $return;
	}

	public function get_iterable_contact($email)
	{
		$return = [];
		$contact = $this->get("users/$email");

		if (empty((array)$contact)) {
			global $current_user;
			wp_get_current_user();

			$contact = $this->create_iterable_contact(sanitize_email($email));

			if ($contact->code == "Success") {
				$return = [
					'first_name' => $current_user->first_name,
					'last_name'  => $current_user->last_name,
					'email'      => $email
				];

				$return['lists'] = [];
				$return['unsubscribedChannelIds'] = [];
			}
		} else {
			if (!is_null($contact)) {
				$return = [
					'id'         => $contact->user->userId ?? "",
					'first_name' => $contact->user->dataFields->firstName ?? "",
					'last_name'  => $contact->user->dataFields->lastName ?? "",
					'email'      => $contact->user->email ?? ""
				];

				$return['lists'] = $contact->user->dataFields->subscribedMessageTypeIds ?? "";
				$return['unsubscribedChannelIds'] = $contact->user->dataFields->unsubscribedChannelIds ?? "";
			}
		}

		return $return;
	}

	public function create_contact($email)
	{
		$contact = $this->post('contacts', ['contact' => ['email' => $email]]);

		return $contact;
	}

	public function create_iterable_contact($email, $first_name = "", $last_name = "")
	{
		$contact = $this->post('users/update', [
			"email" => $email,
			"dataFields" => [
				"firstName" => $first_name,
				"lastName" => $last_name
			],
			"preferUserId" => true,
			"mergeNestedObjects" => true
		]);

		return $contact;
	}

	public function update_contact($data)
	{
		$contact = $this->post('contact/sync', [
			'contact' => [
				'email'     => $data['email'],
				'firstName' => $data['first_name'],
				'lastName'  => $data['last_name']
			]
		]);

		return $contact;
	}

	public function update_iterable_contact($data)
	{
		$contact = $this->post('users/update', [
			'email'     => $data['email'],
			'userId'     => $data['id'],
			"dataFields" => [
				'firstName' => $data['first_name'],
				'lastName'  => $data['last_name']
			],
			"preferUserId" => true,
			"mergeNestedObjects" => true
		]);

		return $contact;
	}

	public function update_channels($data)
	{
		return $this->post('users/updateSubscriptions', $data);
	}

	public function update_contact_lists($data)
	{
		return $this->post('contactLists', ['contactList' => $data]);
	}

	public function validate_credentials()
	{
		switch ($this->centre_type) {
			case 'iterable':
				$api = $this->api('campaigns/metrics');
				break;
			default:
				$api = $this->api('');
				break;
		}

		$url = $api['url'];

		$response = wp_remote_get($url, $api['args']);
		if (is_wp_error($response)) {
			return false;
		}

		if ($response['response']['code'] === 200) {
			return true;
		}

		return false;
	}

	protected function get($endpoint, $params = array())
	{

		$api = $this->api($endpoint);
		$url = $api['url'];

		if (count($params) > 0) {
			$url = $api['url'] . '?' . http_build_query($params);
		}

		$response = wp_remote_get($url, $api['args']);
		$body     = wp_remote_retrieve_body($response);
		return json_decode($body);
	}

	protected function post($endpoint, $params = array())
	{

		$api = $this->api($endpoint);
		$url = $api['url'];

		$args = array_merge($api['args'], ['body' => json_encode($params)]);
		$response = wp_remote_post($url, $args);
		$body     = wp_remote_retrieve_body($response);

		return json_decode($body);
	}

	protected function api($endpoint)
	{
		$ac_url     	= esc_attr(get_option($this->plugin_name . '_url'));
		$ac_key     	= esc_attr(get_option($this->plugin_name . '_key'));
		$ac_version = '/api/3/';
		if (!empty($this->email_centers) && !empty($this->email_centers[$this->centre_type]) && !empty($this->email_centers[$this->centre_type]['route'])) {
			$ac_version = $this->email_centers[$this->centre_type]['route'];
		}

		$url = $ac_url . $ac_version . $endpoint;

		if (strpos($endpoint, $ac_url) !== false) {
			$url = $endpoint;
		}

		$args = ['headers' => [$this->key_name => $ac_key]];

		return array('args' => $args, 'url' => $url);
	}

	private function load_dependencies()
	{

		require_once plugin_dir_path(dirname(__FILE__)) . 'includes/loader.php';

		require_once plugin_dir_path(dirname(__FILE__)) . 'admin/admin.php';
		require_once plugin_dir_path(dirname(__FILE__)) . 'public/public.php';

		$this->loader = new Universal_email_preference_center_Loader();
	}

	private function define_admin_hooks()
	{

		$plugin_admin = new Universal_email_preference_center_Admin($this->get_plugin_name(), $this->get_version());

		$this->loader->add_action('admin_menu', $plugin_admin, 'add_plugin_options');
		$this->loader->add_action('admin_init', $plugin_admin, 'options_init');
		$this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts');
		$this->loader->add_filter('plugin_action_links_universal-email-preference-center/universal-email-preference-center.php', $plugin_admin, 'plugin_settings_link', 10, 2);
	}

	private function define_public_hooks()
	{

		$plugin_public = new Universal_email_preference_center_Public($this->get_plugin_name(), $this->get_version());

		$this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_scripts');
		$this->loader->add_action('init', $plugin_public, 'init');
		$this->loader->add_action('wp_ajax_update_preferences', $plugin_public, 'ajax_update_preferences');
		$this->loader->add_action('wp_ajax_nopriv_update_preferences', $plugin_public, 'ajax_update_preferences');
		$this->loader->add_action('wp_head', $plugin_public, 'custom_styles');
	}
}
