<?php

class Universal_email_preference_center_Public
{
    private  $plugin_name;
    private  $version;
    public  $centre_type;
    public $premium;
    public function __construct($plugin_name, $version)
    {
        $this->plugin_name = $plugin_name;
        $this->version = $version;
        $this->centre_type = esc_attr(get_option($this->plugin_name . '_centre_type'));

        if ( defined("UEPC_PREMIUM_DIR_PATH") && file_exists( UEPC_PREMIUM_DIR_PATH.'/public/public.php' ) ) {
            require_once UEPC_PREMIUM_DIR_PATH . '/public/public.php';
            $this->premium = new Universal_email_preference_center_Public_Premium($this->plugin_name);
        }
    }
    
    public function init()
    {
        $this->register_shortcodes();
    }
    
    public function register_shortcodes()
    {
        if (function_exists("uepcp_fs") && uepcp_fs()->is__premium_only() && uepcp_fs()->can_use_premium_code() && !empty($this->premium)) {
            add_shortcode('universal-email-preference-center', array($this, 'preference_center_shortcode_premium'));
        }else{
            add_shortcode('universal-email-preference-center', array($this, 'preference_center_shortcode'));
        }
    }
    
    public function enqueue_scripts()
    {
        wp_enqueue_script(
            $this->plugin_name,
            plugin_dir_url(__FILE__) . 'js/public.js',
            array('jquery'),
            $this->version,
            false
        );
        $string_success_text = get_option($this->plugin_name . '_string_success_text', esc_attr_x('Your email preferences have been saved.', 'Success string when subscribing to list', 'universal-email-preference-center'));
        $string_error_text = get_option($this->plugin_name . '_string_error_text', esc_attr_x('There was an error processing your request.  Try again.', 'Error string when subscribing to list', 'universal-email-preference-center'));
        $string_saving_text = get_option($this->plugin_name . '_string_saving_text', esc_attr_x('Please wait...', 'Text when save preferences button is pressed', 'universal-email-preference-center'));
        $button_save_preferences = get_option($this->plugin_name . '_button_save_preferences', esc_attr_x('Save Preferences', 'Button text if email has NOT been retrieved', 'universal-email-preference-center'));
        $tamper_protection = filter_var(get_option($this->plugin_name . '_tamper_protection'), FILTER_VALIDATE_BOOLEAN);
        wp_localize_script($this->plugin_name, 'ac_ajax', [
            'ajax_url'                 => admin_url('admin-ajax.php'),
            'update_nonce'             => wp_create_nonce('update_nonce'),
            'success_text'             => $string_success_text,
            'error_text'               => $string_error_text,
            'saving_text'              => $string_saving_text,
            'save_text'                => $button_save_preferences,
            'enable_tamper_protection' => $tamper_protection,
        ]);
    }
    
    public function preference_center_shortcode_premium($atts)
    {
        return $this->premium->universal_email_preference_center_premium_callback($atts, $this, plugin_dir_path(dirname(__FILE__)));
    }
    public function preference_center_shortcode($atts)
    {
        ob_start();
        $args = shortcode_atts(array(
            'show_action_text'       => false,
            'show_list_descriptions' => true,
            'show_welcome_text'      => true,
            'list_ids'               => null,
        ), $atts, 'universal-email-preference-center');
        $show_action_text = filter_var($args['show_action_text'], FILTER_VALIDATE_BOOLEAN);
        $show_list_descriptions = filter_var($args['show_list_descriptions'], FILTER_VALIDATE_BOOLEAN);
        $show_welcome_text = filter_var($args['show_welcome_text'], FILTER_VALIDATE_BOOLEAN);
        $sending_email = false;
        $use_wordpress_user = false;
        $button_save_preferences = get_option($this->plugin_name . '_button_save_preferences', esc_attr_x('Save Preferences', 'Button text if email has NOT been retrieved', 'universal-email-preference-center'));
        $button_subscribe = get_option($this->plugin_name . '_button_subscribe', esc_attr_x('Subscribe', 'Button text if email HAS been retrieved.', 'universal-email-preference-center'));
        
        $email = $this->getEmailFromGet();

        if (empty($email) && isset($use_wordpress_user) && is_user_logged_in()) {
            $current_user = wp_get_current_user();
            $email = sanitize_email($current_user->user_email);
        }
        
        $uepc = new Universal_email_preference_center();
        $contact = $uepc->validate_credentials();
        if ($contact) {
            $contact = $uepc->get_contact($email);
            $subscriber_id = (!empty($contact['id']) ?: $contact['id']);
            $lists = $this->get_lists();
            $has_credentials = $this->has_credentials();
            $unsubscribed_channels = isset($contact['unsubscribedChannelIds']) ? $contact['unsubscribedChannelIds'] : [];
            $subscribed_lists = [];
            $channel_ids = [];
            foreach ($lists as $list) {
                if (isset($list['channel']['id'])) {
                    $channel_ids[] = $list['channel']['id'];
                }
            }
            $channel_ids = array_unique($channel_ids);
            if (!is_null($email) && isset($contact['lists']) && !empty($contact['lists'])) {
                $subscribed_lists = $contact['lists'];
            }
            require plugin_dir_path(dirname(__FILE__)) . 'public/partials/preference-center.php';
        }else{
            uepc_log($this->centre_type, "The connection failed, or the plugin isn't set up yet.", 'notice');
            echo "<h5 class='uepc-error'>The connection failed, or the plugin isn't set up yet.</h5>";
        }
        return ob_get_clean();
    }
    
    public function get_lists()
    {
        $lists = get_option($this->plugin_name . '_list_data');
        $lists = (!$lists ? array() : $lists);
        return $lists;
    }
    
    public function filter_iterable_list_reverse_enabled_ids($submitted_ids)
    {
        $ids['subscribed'] = [];
        $ids['unsubscribed'] = [];
        $lists = $this->get_lists();
        foreach ($lists as $id => $item) {
            if (in_array($id, $submitted_ids) && !$item['value_reverse'] || !in_array($id, $submitted_ids) && $item['value_reverse']) {
                $ids['subscribed'][] = $id;
            }else{
                $ids['unsubscribed'][] = $id;
            }
        }
        return $ids;
    }
    
    public function has_credentials()
    {
        $creds = get_option($this->plugin_name . '_key');
        $creds = get_option($this->plugin_name . '_url');
        
        if (!$creds) {
            return false;
        } else {
            return true;
        }
    }
    
    public function get_subscribed_lists($contact)
    {
        $subscribed_lists = array();
        if (is_object($contact) && $contact->result_code != 0) {
            foreach ($contact->lists as $list) {
                if (isset($list->status) && $list->status == 1) {
                    $subscribed_lists[] = $list->listid;
                }
            }
        }
        return $subscribed_lists;
    }
    
    public function ajax_update_preferences()
	{
		if (check_ajax_referer('update_nonce', 'security')) {

			$tamper_protection = (bool) get_option($this->plugin_name . '_tamper_protection');
			$enable_name       = (bool) get_option($this->plugin_name . '_enable_name');

			parse_str($_POST['data'], $data);
			$current_subscribed_list_ids = unserialize(urldecode($data['current_subscribed_list_ids']));
			$all_channel_ids = unserialize(urldecode($data['all_channel_ids']));
			if (!isset($data['subscribed_list_ids'])) {
				$data['subscribed_list_ids'] = array();
			}
			$submitted_list_ids = isset($data['subscribed_list_ids']) && !empty($data['subscribed_list_ids']) ? $data['subscribed_list_ids'] : [];
			$submitted_channel_ids = isset($data['subscribed_channel_ids']) && !empty($data['subscribed_channel_ids']) ? $data['subscribed_channel_ids'] : [];
            
            $subscribed_channel_ids = array_values( array_diff($all_channel_ids, $submitted_channel_ids) );

			$upec = new Universal_email_preference_center();

			$contact = $upec->get_contact(sanitize_email($data['email']));

			if (function_exists("uepcp_fs") && uepcp_fs()->is__premium_only() && uepcp_fs()->can_use_premium_code() && $enable_name) {
                if ($enable_name) {
                    $this->premium->uepc_update_user_details($upec, $contact, $data);
                }
			}

			$lists = [];

			/**
			 * Create array of our current subscribed lists and statuses
			 */
			foreach ($current_subscribed_list_ids as $current_subscribed_list_id) {
				$status                               = in_array($current_subscribed_list_id, $submitted_list_ids) ? 1 : 2;  //1 = subscribed 2 = un-subscribed
				$lists[$current_subscribed_list_id] = $status;
			}

			/**
			 * Loop through our submitted list IDs and mark them down as subscribed.
			 */

			foreach ($submitted_list_ids as $submitted_list_id) {
				$lists[$submitted_list_id] = 1;
			}

			$submitted_list_ids_filter = array_map(function ($item) {
				return (int)$item;
			}, $submitted_list_ids);

            $subscribed_channel_ids_filter = array_map(function ($item) {
				return (int)$item;
			}, $subscribed_channel_ids);
            
			switch ($this->centre_type) {
				case 'iterable':
					$filter_ids = $this->filter_iterable_list_reverse_enabled_ids($submitted_list_ids_filter);

					$update_data = [
						"email" => $contact['email'],
						"userId" => $contact['id'],
						"subscribedMessageTypeIds" => $filter_ids['subscribed'],
						"unsubscribedMessageTypeIds" => $filter_ids['unsubscribed'],
						"unsubscribedChannelIds" => $subscribed_channel_ids_filter
					];

					if (isset($data['templateId']) && !empty($data['templateId']) ) {
						$update_data['templateId'] = (int)$data['templateId'];
					}

					if (isset($data['campaignId']) && !empty($data['campaignId']) ) {
						$update_data['campaignId'] = (int)$data['campaignId'];
					}
					
					$sync = $upec->update_channels($update_data);
					break;
				default:
					foreach ($lists as $id => $status) {
						$sync = $upec->update_contact_lists([
							'list'    => $id,
							'contact' => $contact['id'],
							'status'  => $status
						]);
					}
					break;
			}

			switch ($this->centre_type) {
				case 'iterable':
					if ($sync->code == 'Success') {
						uepc_log($this->centre_type, "User list update successfully.<br>".json_encode($sync), 'success');
					}else {
						uepc_log($this->centre_type, "Error updating list status.<br>".json_encode($sync), 'error');
						wp_send_json('Error updating list status.',500);
					}
					break;
				default:
					if (empty($sync->contacts)) {
						uepc_log($this->centre_type, "Error updating list status.<br>".json_encode($sync), 'error');
						wp_send_json('Error updating list status.', 500);
					}else{
						uepc_log($this->centre_type, "User list update successfully.<br>".json_encode($sync), 'success');
					}
					break;
			}

			//We re grab the contact info from AC to include newly subscribed lists.
			$contact = $upec->get_contact(sanitize_email($data['email']));
			wp_send_json(urlencode(serialize($contact['lists'])), 200);
		}
		die();
	}
    
    public function getEmailFromGet()
    {
        if (isset($_GET['email']) && !empty($_GET['email']) && sanitize_email($_GET['email'])) {
            /** Since we are using $_GET the PHP will convert + automatically to a blank space when coming from $_GET
             * In order to handle GMAIL alias addresses we need to convert those spaces back to + */
            $handle_plus = str_replace( ' ', '+', sanitize_email( $_GET['email'] ) );
            return $handle_plus;
        }
        
        return null;
    }
    
    protected function get_subscriber_id_from_email($email)
    {
        $upec = new Universal_email_preference_center();
        $subscriber_id = null;
        $contact = $upec->get_contact($email);
        
        if (is_array($contact)) {
        } elseif(is_object($contact)) {
            if (isset($contact->subscriberid)) {
                return $contact->subscriberid;
            }
        }
        
        return false;
    }
    
    protected function is_valid_subscriber_id($subscriber_id)
    {
        if (isset($_GET['id']) && !empty($_GET['id']) && is_numeric($_GET['id'])) {
            if ($_GET['id'] == $subscriber_id) {
                return true;
            }
        }
        return false;
    }
    
    protected function get_contact_id($email)
    {
        $upec = new Universal_email_preference_center();
        $contact = $upec->get_contact(sanitize_email($email));
        return $contact['id'];
    }

    public function custom_styles(){
        if ((int)get_option($this->plugin_name . '_css_enable')) {
            echo '<style type="text/css">'.get_option($this->plugin_name . '_css_code').'</style>';
        }
    }
}
